package ap.proof.tree;

import ap.terfor.TermOrder
import ap.terfor.conjunctions.Conjunction
import ap.util.Debug

object ProofTree {
  
  private val AC = Debug.AC_PROOF_TREE
  
}

trait ProofTree {

  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(ProofTree.AC,
                   (order isSortingOf closingConstraint) &&
                   (constantFreedom freeConstsAreUniversal bindingContext) &&
                   (closingConstantFreedom freeConstsAreUniversal bindingContext) &&
                   closingConstantFreedom <= constantFreedom)
  //////////////////////////////////////////////////////////////////////////////

  val subtrees : Seq[ProofTree]
  
  /**
   * The fully simplified closing constraint
   */
  val closingConstraint : Conjunction

  /**
   * The constants that can be considered free (resp., that have to be
   * considered non-free) in this proof tree.
   */
  val closingConstantFreedom : ConstantFreedom
  
  /**
   * <code>true</code> if it is possible to apply rules to any goal in this
   * tree
   */
  val stepPossible : Boolean
  
  /**
   * <code>true</code> if there are chances that the
   * <code>closingConstraint</code> of this tree changes by applying rules
   * to any goal
   */
  val stepMeaningful : Boolean
  
  /**
   * <code>true</code> if the sets of free constants have reached a fixed point
   */
  val fixedConstantFreedom : Boolean
  
  /**
   * the vocabulary available at a certain node in the proof
   */
  val vocabulary : Vocabulary
  
  def order : TermOrder = vocabulary.order
  def bindingContext : BindingContext = vocabulary.bindingContext
  def constantFreedom : ConstantFreedom = vocabulary.constantFreedom
  
}
