package ap.terfor;

import scala.collection.immutable.HashSet

import ap.util.{Debug, Logic, APTestCase, PlainRange}
import ap.basetypes.IdealInt
import linearcombination.LinearCombination

class TestTermOrder(n : String) extends APTestCase(n) {

  def runTest = {
    n match {
      case "testSimpleExtension" => testSimpleExtension
      case "testConditionalExtension" => testConditionalExtension
      case "testLinearCombinationComparison" => testLinearCombinationComparison
    }
  }
  
  private val consts = for (i <- Array.range(0, 20)) yield new ConstantTerm("c" + i)

  def testSimpleExtension = {
    var to = TermOrder.EMPTY
    for (c <- consts) to = to.extend(c, Set.empty)
    
    for ((i, j) <- (Debug.randoms(0, 20).take(100) zip
                    Debug.randoms(0, 20).take(100))) {
      assertEquals(Debug.signum(i compare j),
                   Debug.signum(to.compare(consts(i), consts(j))))
      assertEquals(Debug.signum(j compare i),
                   Debug.signum(to.compare(VariableTerm(i), VariableTerm(j))))
      assertTrue(to.compare(consts(i), VariableTerm(4))
                 < 0)
      assertTrue(to.compare(consts(i), OneTerm)
                 > 0)
    }
  }

  def testConditionalExtension = {
    var to = TermOrder.EMPTY
    for (c <- consts) {
      val biggerConsts = HashSet.empty ++
                         (for (i <- Debug.randoms(0, 20).take(Debug.random(0,50)))
                          yield consts(i))
      to = to.extend(c, biggerConsts - c)
    }
  }

  def testLinearCombinationComparison = {
    var to = TermOrder.EMPTY
    for (c <- consts) to = to.extend(c, Set.empty)

    val constTerms = consts ++ List(OneTerm)
  
    def randomInput = for (i <- PlainRange(10, constTerms.size))
                      yield (IdealInt(Debug.random(-3, 3)), constTerms(i))

    // not really a correct implementation, but it covers all cases that occur
    // here
    def smallerThan(lc1 : LinearCombination, lc2 : LinearCombination) : Boolean =
      Logic.exists(0, 10,
                   (i:Int) => Logic.forall(0, i,
                                           (j:Int) => lc1(j) == lc2(j)) &&
                              (to.compare(lc1(i) _2, lc2(i) _2) < 0 ||
                               (lc1(i) _2) == (lc2(i) _2) &&
                               ((lc1(i) _1) compareAbs (lc2(i) _1)) < 0))

    for (_ <- PlainRange(100)) {
      val lc1 = LinearCombination(randomInput, to)
      val lc2 = LinearCombination(randomInput, to)

      val res = to.compare(lc1, lc2)
      if (smallerThan(lc1, lc2))
        assertTrue ( res < 0 )
      else if (smallerThan(lc2, lc1))
        assertTrue ( res > 0 )
    }
  }
}
