package ap.util;

object LazyMappedMap {
  
  private val AC = Debug.AC_MAP_UTILS
  
}

/**
 * Transform a <code>Map m</code> by composing it with two functions into a map
 * <code>valueMapping . m . keyUnmapping</code>. <code>keyMapping</code> has to
 * be the inverse of <code>keyUnmapping</code>, and has to be injective
 */
class LazyMappedMap[A,B,C,D] (oriMap : scala.collection.Map[A,B],
                              keyMapping : (A) => C,
                              keyUnmapping : PartialFunction[C, A],
                              valueMapping : (B) => D)
      extends scala.collection.Map[C,D] {

  def get(key : C) : Option[D] =
    if (keyUnmapping isDefinedAt key) {
      val oriKey = keyUnmapping(key)
      //////////////////////////////////////////////////////////////////////////
      Debug.assertInt(LazyMappedMap.AC, keyMapping(oriKey) == key)
      //////////////////////////////////////////////////////////////////////////
      (oriMap get oriKey) map valueMapping
    } else {
      None
    }
      
  def size : Int = oriMap.size
  
  def elements : Iterator[(C,D)] =
    for ((key, value) <- oriMap.elements) yield {
      val mappedKey = keyMapping(key)
      val mappedValue = valueMapping(value)
      //////////////////////////////////////////////////////////////////////////
      Debug.assertInt(LazyMappedMap.AC, (keyUnmapping isDefinedAt mappedKey) &&
                                        keyUnmapping(mappedKey) == key)
      //////////////////////////////////////////////////////////////////////////      
      (mappedKey, mappedValue)
    }
  
}
