package ap.util;

object PlainRange {
  def apply(end : Int) : PlainRange =
    new IntervalPlainRange(0, end)
    
  def apply(start : Int, end : Int) : PlainRange =
    new IntervalPlainRange(start, end)
}

/**
 * Extremely simple class for iterating over intervals of integers
 */
abstract class PlainRange extends Seq[Int]

protected class IntervalPlainRange(start : Int, end : Int) extends PlainRange {

  override def foreach(f: Int => Unit) : Unit = {
    var i : Int = start
    while (i < end) {
      f(i)
      i = i+1
    }
  }
  
  override def filter(pred : Int => Boolean) : PlainRange =
    new PredicatedPlainRange(start, end, pred)
  
  def length = end - start
  
  def elements = new Iterator[Int] {
    private var i : Int = start
    def hasNext = (i < end)
    def next = {
      val res = i
      i = i+1
      res
    }
  }

  def apply(n : Int) = (n + start)
  
}

protected class PredicatedPlainRange(start : Int, end : Int, pred : Int => Boolean)
      extends IntervalPlainRange(start, end) {

  override def foreach(f: Int => Unit) : Unit = {
    var i : Int = start
    while (i < end) {
      if (pred(i)) f(i)
      i = i+1
    }
  }

  override def filter(pred2 : Int => Boolean) : PlainRange =
    new PredicatedPlainRange(start, end, (i) => pred(i) && pred2(i))

  override def length = throw new Predef.UnsupportedOperationException

  override def apply(n : Int) = throw new Predef.UnsupportedOperationException
}