package ap.util;

object UnionMap {
  
  private val AC = Debug.AC_MAP_UTILS

  def apply[A, B](map1 : scala.collection.Map[A, B],
                  map2 : scala.collection.Map[A, B]) : scala.collection.Map[A, B] =
    if (map1.size == 0)
      map2
    else if (map2.size == 0)
      map1
    else
      new UnionMap (map1, map2)

}

/**
 * A (lazy) <code>Map</code> that represents the union of two <code>Map</code>s with
 * disjoint key domains
 */
class UnionMap[A, B] private (map1 : scala.collection.Map[A, B],
                              map2 : scala.collection.Map[A, B])
      extends scala.collection.Map[A, B] {

  def get(t : A) : Option[B] = (map1 get t) match {
    case x@Some(_) => x
    case None => map2 get t
  }
  
  def size : Int = {
    ////////////////////////////////////////////////////////////////////////////
    Debug.assertInt(UnionMap.AC, Seqs.disjoint(map1.keySet, map2.keySet))
    ////////////////////////////////////////////////////////////////////////////
    map1.size + map2.size
  }
  
  def elements : Iterator[(A, B)] = map1.elements ++ map2.elements
  
}
